from dataclasses import dataclass

@dataclass
class couch:
  name = "couch"
  object_associations = {}
  duplicate = 0.3

@dataclass
class tv_set:
  name = "tv_set"
  object_associations = {}
  duplicate = 0.0

@dataclass
class bookshelf:
	name = "bookshelf"
	object_associations = {}
	duplicate = 0.1

@dataclass
class coffee_table:
	name = "coffee_table"
	object_associations = {}
	duplicate = 0.1
        
@dataclass
class fridge:
	name = "fridge"
	object_associations = {}
	duplicate = 0.2
        
@dataclass
class chair:
	name = "chair"
	object_associations = {}
	duplicate = 0.8

@dataclass
class cupboard:
	name = "cupboard"
	object_associations = {}
	duplicate = 0.9

@dataclass
class counter_top:
	name = "counter_top"
	object_associations = {}
	duplicate = 0.5

@dataclass
class dish_washer:
	name = "dish_washer"
	object_associations = {}
	duplicate = 0.0

@dataclass
class bed:
  name = "bed"
  object_associations = {}
  duplicate = 0.2
  
@dataclass
class desk:
  name = "desk"
  object_associations = {}
  duplicate = 0.1
  
@dataclass
class nightstand:
  name = "nightstand"
  object_associations = {}
  duplicate = 0.2
  
@dataclass
class toilet:
  name = "toilet"
  object_associations = {}
  duplicate = 0.0
  
@dataclass
class bathtub:
  name = "bathtub"
  object_associations = {}
  duplicate = 0.0
  
@dataclass
class shelf:
  name = "shelf"
  object_associations = {}
  duplicate = 0.7
  
@dataclass
class washing_machine:
  name = "washing_machine"
  object_associations = {}
  duplicate = 0.0
  
@dataclass
class table:
  name = "table"
  object_associations = {}
  duplicate = 0.0